/********************************************************************
 FileName:    	inputs.c
 Dependencies: 	aguijon.h
 Hardware:	Aguijon rev3.0 & rev4.0
 Complier:  	Microchip XC16, C30
 Company:	Vinagrn Digital
 Author:	Juan Carlos Orozco Gutierrez

 Software License Agreement:

 This software has been licensed under the GNU General Public
 License is intended to guarantee your freedom to share and change free
 software--to make sure the software is free for all its users.

********************************************************************
 File Description:
    Input Support File


 Change History:
  Rev   Description                                 Modified by:
  ----  -----------------------------------------  --------------
  1.0   Initial release                             JCOG
  2.0   Updated with [VD] HammerHead libraries      JCOG
  2.1   Added support for Aguijon PRO               JCOG

 For extra documentation and support:
 *  http://www.vinagrondigital.com
********************************************************************/
#include "aguijon.h"
#include "inputs.h"




///////////////////////////////////////
static const unsigned char DipSWkeyTable[16] = {15,7,11,3,13,5,9,1,14,6,10,2,12,4,8,0};

/*DIP SW*/
///////////////////////////////////////////////////////////////////////////////
void DIPSW_Config(int config)
{
    switch(config){
        case DIPSW_ENABLE_MSB_CMD:
            DIPSW_ENABLE_MSB_PIN = 0;
            DIPSW_ENABLE_LSB_PIN = 1;
            break;
        case DIPSW_ENABLE_LSB_CMD:
            DIPSW_ENABLE_MSB_PIN = 1;
            DIPSW_ENABLE_LSB_PIN = 0;
            break;
        case DIPSW_DISABLE_CMD:
            DIPSW_ENABLE_LSB_PIN = 1;
            DIPSW_ENABLE_MSB_PIN = 1;
            break;
    }
}

unsigned char DIPSW_Read(void)
{
    unsigned char lsbValue = 0;
    unsigned char msbValue = 0;
    unsigned char newValue = 0;

#if defined(AGUIJON_REV_3)
    unsigned char rawValue = 0;
    /*PORTB is 16 bits wide, we only care about these bits:
     * x=don't care, 1=care
     *
     * PORTB: xx11 11xx xxxx xxxx
     *
     * The nibbles are swapped, meaning that a 0x8 on both nibbles, is equivalent
     * to 0x6, hence the lookup table above (at the beggining of the file)
     */

    DIPSW_Config(DIPSW_ENABLE_LSB_CMD); //remember that DIP SW is shared
    rawValue = (PORTB & KEYBOARD_MASK) >> 10;   //construct LSB nibble
    lsbValue = DipSWkeyTable[rawValue]; 
    
    DIPSW_Config(DIPSW_ENABLE_MSB_CMD); 
    rawValue = (PORTB & KEYBOARD_MASK) >> 10;   //construct MSB nibble
    msbValue = DipSWkeyTable[rawValue];

    DIPSW_Config(DIPSW_DISABLE_CMD);

    newValue  = (newValue | msbValue) << 4;     //final shifting work
    newValue |= lsbValue;                       //we have a byte
#elif defined(AGUIJON_REV_4)
    DIPSW_Config(DIPSW_ENABLE_LSB_CMD); //remember that DIP SW is shared
    lsbValue = (PORTB & KEYBOARD_MASK) >> 10;   //construct LSB nibble

    DIPSW_Config(DIPSW_ENABLE_MSB_CMD);
    msbValue = (PORTB & KEYBOARD_MASK) >> 6;   //construct MSB nibble

    DIPSW_Config(DIPSW_DISABLE_CMD);

    newValue  = (newValue | msbValue);     //final shifting work
    newValue |= lsbValue;                  //we have a byte
    newValue ^= 0xFF;
#endif

    return newValue;    //done!
}
///////////////////////////////////////////////////////////////////////////////

/*keyboard switches*/
///////////////////////////////////////////////////////////////////////////////
unsigned char SW_Read(void)
{
    static unsigned char pressedKey    = 0;
    static unsigned char lastPressed   = 0;
    unsigned char        portData      = 0;
    int                  i, scannedKey = 0;

    //1.Read keyboard
    portData = (PORTB & KEYBOARD_MASK) >> 10; //SWITCH1 will be read first

#if defined(AGUIJON_REV_3)
    for(i=0 ; i<4 ; i++){
        if(!(portData & (1<<i))){   //check if bit is not set
            pressedKey = 4-i;       //reverse logic
            scannedKey = i;
            break;
        }
    }
#elif defined(AGUIJON_REV_4)
    for(i=0 ; i<4 ; i++){
        if(!(portData & (1<<i))){   //check if bit is not set
            pressedKey = i+1;
            scannedKey = i;
            break;
        }
    }
#endif

    //2.Debounce if a valid SW was detected
    if(pressedKey){
        delayms(1);                                 //debounce delay
        portData = (PORTB & KEYBOARD_MASK) >> 10;   //read PORT *again*
        if((portData & (1<<scannedKey))){           //check if bit is set
            pressedKey = 0;                         //SW's are active low
        }
    }

    //3.State Logic
    if(pressedKey){                         //pressed key?
        if(pressedKey == lastPressed){      //same as before?
            return 0;                       //not good
        }else{
            lastPressed = pressedKey;       //update lastpressed
        }
    }else if(lastPressed != 0){
        lastPressed = 0;
    }

    return pressedKey;  //done!
}
///////////////////////////////////////////////////////////////////////////////

/*Opto isolated inputs*/
///////////////////////////////////////////////////////////////////////////////
unsigned char IN_Read(void)
{
    static unsigned char activeInput = 0;
    static unsigned char lastActive  = 0;
    unsigned char        portData    = 0;
    int                  i,scannedIN = 0;

#if defined(AGUIJON_REV_3)
    //1.Read inputs
    portData = (PORTB & INPUT_MASK);    //INPUT1 will be read first

    for(i=0 ; i<4 ; i++){
        if(!(portData & (1<<i))){   //check if bit is not set
            activeInput = i+1;
            scannedIN   = i;
            break;
        }
    }

    //2.Debounce if a valid SW was detected
    if(activeInput){
        delayms(1);                     //debounce delay
        portData = PORTB & INPUT_MASK;  //read PORT *again*
        if((portData & (1<<scannedIN))){//check if bit is set
            activeInput = 0;            //IN's are active low (for this function)
        }
    }
#elif defined(AGUIJON_REV_4)
    //1.Read inputs
    portData  =  (PORTA & INPUT1_MASK) >> 9;     //get input 1
    portData |= ((PORTE & INPUT2_MASK) >> 8);    //get input 2
    portData |= ((PORTE & INPUT3_MASK) >> 6);    //get input 3
    portData |=  (PORTC & INPUT4_MASK);          //get input 4

    for(i=0 ; i<4 ; i++){
        if(!(portData & (1<<i))){   //check if bit is not set
            activeInput = i+1;
            scannedIN   = i;
            break;
        }
    }

    //2.Debounce if a valid SW was detected
    if(activeInput){
        delayms(1);                     //debounce delay

        //read inputs *again*
        portData  =  (PORTA & INPUT1_MASK) >> 9;     //get input 1
        portData |= ((PORTE & INPUT2_MASK) >> 8);    //get input 2
        portData |= ((PORTE & INPUT3_MASK) >> 6);    //get input 3
        portData |=  (PORTC & INPUT4_MASK);          //get input 4

        if((portData & (1<<scannedIN))){//check if bit is set
            activeInput = 0;            //IN's are active low (for this function)
        }
    }
#endif

    //3.State Logic
    if(activeInput){                    //active input?
        if(activeInput == lastActive){  //same as before?
            return 0;                   //not good
        }else{
            lastActive = activeInput;   //update lastactive
        }
    }else if(lastActive != 0){
        lastActive = 0;
    }

    return activeInput;  //done!
}
///////////////////////////////////////////////////////////////////////////////

/*Port functions*/
///////////////////////////////////////////////////////////////////////////////
unsigned char SW_ReadPort(void)
{
    //returns raw port data (ex.0x08)
    //inputs are active 0!
    unsigned char keybPortValue = 0;
    unsigned char portData      = 0;

    portData = (PORTB & 0x3C00) >> 10;
#if defined(AGUIJON_REV_3)
    keybPortValue = DipSWkeyTable[portData];
#endif

    return keybPortValue;
}

unsigned char IN_ReadPort(void)
{
    //returns raw port data (ex.0x04)
    //inputs are active 0!
    unsigned char inputPortValue = 0;

#if defined(AGUIJON_REV_3)
    inputPortValue  = PORTB & 0x000F;
    inputPortValue ^= 0xFF; //user should mask when comparing
#elif defined(AGUIJON_REV_4)
    inputPortValue  =  (PORTA & INPUT1_MASK) >> 9;     //get input 1
    inputPortValue |= ((PORTE & INPUT2_MASK) >> 8);    //get input 2
    inputPortValue |= ((PORTE & INPUT3_MASK) >> 6);    //get input 3
    inputPortValue |=  (PORTC & INPUT4_MASK);          //get input 4
#endif

    return inputPortValue;
}
///////////////////////////////////////////////////////////////////////////////
